@@ -5,10 +5,11 @@ from enum import Enum
 from typing import Optional
 
 import kubernetes
+from kubernetes.client.models import V1JobStatus
 
 from dagster import DagsterInstance
 from dagster import _check as check
-from dagster._core.storage.pipeline_run import PipelineRunStatus
+from dagster._core.storage.pipeline_run import DagsterRunStatus
 
 DEFAULT_WAIT_TIMEOUT = 86400.0  # 1 day
 DEFAULT_WAIT_BETWEEN_ATTEMPTS = 10.0  # 10 seconds
@@ -136,13 +137,13 @@ class DagsterKubernetesClient:
         self.timer = timer
 
     @staticmethod
-    def production_client():
+    def production_client(batch_api_override=None):
         return DagsterKubernetesClient(
-            kubernetes.client.BatchV1Api(),
-            kubernetes.client.CoreV1Api(),
-            logging.info,
-            time.sleep,
-            time.time,
+            batch_api=batch_api_override or kubernetes.client.BatchV1Api(),
+            core_api=kubernetes.client.CoreV1Api(),
+            logger=logging.info,
+            sleeper=time.sleep,
+            timer=time.time,
         )
 
     ### Job operations ###
@@ -315,14 +316,10 @@ class DagsterKubernetesClient:
 
             # Reads the status of the specified job. Returns a V1Job object that
             # we need to read the status off of.
-            status = None
-
-            def _get_job_status():
-                job = self.batch_api.read_namespaced_job_status(job_name, namespace=namespace)
-                return job.status
-
-            status = k8s_api_retry(
-                _get_job_status, max_retries=3, timeout=wait_time_between_attempts
+            status = self.get_job_status(
+                job_name=job_name,
+                namespace=namespace,
+                wait_time_between_attempts=wait_time_between_attempts,
             )
 
             # status.succeeded represents the number of pods which reached phase Succeeded.
@@ -344,11 +341,23 @@ class DagsterKubernetesClient:
                     raise DagsterK8sPipelineStatusException()
 
                 pipeline_run_status = pipeline_run.status
-                if pipeline_run_status != PipelineRunStatus.STARTED:
+                if pipeline_run_status != DagsterRunStatus.STARTED:
                     raise DagsterK8sPipelineStatusException()
 
             self.sleeper(wait_time_between_attempts)
 
+    def get_job_status(
+        self,
+        job_name: str,
+        namespace: str,
+        wait_time_between_attempts=DEFAULT_WAIT_BETWEEN_ATTEMPTS,
+    ) -> V1JobStatus:
+        def _get_job_status():
+            job = self.batch_api.read_namespaced_job_status(job_name, namespace=namespace)
+            return job.status
+
+        return k8s_api_retry(_get_job_status, max_retries=3, timeout=wait_time_between_attempts)
+
     def delete_job(
         self,
         job_name,
