@@ -1,6 +1,6 @@
 import datetime
 import re
-from typing import Any, Dict
+from typing import Any, Mapping
 
 import kubernetes
 from dateutil.parser import parse
@@ -94,7 +94,7 @@ def _k8s_snake_case_value(val, attr_type, attr_name):
             return k8s_snake_case_dict(klass, val)
 
 
-def k8s_snake_case_dict(model_class, model_dict: Dict[str, Any]):
+def k8s_snake_case_dict(model_class, model_dict: Mapping[str, Any]):
     snake_case_to_camel_case = model_class.attribute_map
     camel_case_to_snake_case = dict((v, k) for k, v in snake_case_to_camel_case.items())
 
@@ -125,8 +125,8 @@ def k8s_snake_case_dict(model_class, model_dict: Dict[str, Any]):
 # Heavily inspired by kubernetes.client.ApiClient.__deserialize_model, with more validation
 # that the keys and values match the expected format. Requires k8s atribute names to be in
 # snake_case.
-def k8s_model_from_dict(model_class, model_dict: Dict[str, Any]):
-    check.dict_param(model_dict, "model_dict")
+def k8s_model_from_dict(model_class, model_dict: Mapping[str, Any]):
+    check.mapping_param(model_dict, "model_dict")
 
     expected_keys = set(model_class.attribute_map.keys())
     invalid_keys = set(model_dict).difference(expected_keys)
